vg.Model = (function() {
  function model() {
    this._defs = null;
    this._data = {};
    this._scene = null;
    this._reset = false;
  }
  
  var prototype = model.prototype;
  
  prototype.defs = function(defs) {
    if (!arguments.length) return this._defs;
    this._defs = defs;
    return this;
  };
  
  prototype.data = function(data) {
    if (!arguments.length) return this._data;

    var tx = this._defs.data.flow || {},
        keys = this._defs.data.defs.map(vg.accessor("name")),
        len = keys.length, i, k;

    for (i=0; i<len; ++i) {
      if (!data[k=keys[i]]) continue;
      this.ingest(k, tx, data[k]);
    }

    return this;
  };
  
  prototype.ingest = function(name, tx, input) {
    this._data[name] = tx[name]
      ? tx[name](input, this._data, this._defs.marks)
      : input;
    this.dependencies(name, tx);
  };
  
  prototype.dependencies = function(name, tx) {
    var source = this._defs.data.source[name],
        data = this._data[name],
        n = source ? source.length : 0, i, x;
    for (i=0; i<n; ++i) {
      x = vg_data_duplicate(data);
      if (vg.isTree(data)) vg_make_tree(x);
      this.ingest(source[i], tx, x);
    }
  };
  
  prototype.width = function(width) {
    if (this._defs) this._defs.width = width;
    if (this._defs && this._defs.marks) this._defs.marks.width = width;
    if (this._scene) this._scene.items[0].width = width;
    this._reset = true;
    return this;
  };
  
  prototype.height = function(height) {
    if (this._defs) this._defs.height = height;
    if (this._defs && this._defs.marks) this._defs.marks.height = height;
    if (this._scene) this._scene.items[0].height = height;
    this._reset = true;
    return this;
  };
  
  prototype.scene = function(node) {
    if (!arguments.length) return this._scene;
    this._scene = node;
    return this;
  };
  
  prototype.build = function() {
    var m = this, data = m._data, marks = m._defs.marks;
    m._scene = vg.scene.build.call(m, marks, data, m._scene);
    m._scene.items[0].width = marks.width;
    m._scene.items[0].height = marks.height;
    m._scene.interactive = false;
    return this;
  };
  
  prototype.encode = function(trans, request, item) {
    if (this._reset) { this.reset(); this._reset = false; }
    var m = this, scene = m._scene, defs = m._defs;
    vg.scene.encode.call(m, scene, defs.marks, trans, request, item);
    return this;
  };

  prototype.reset = function() {
    if (this._scene) {
      vg.scene.visit(this._scene, function(item) {
        if (item.axes) item.axes.forEach(function(axis) { axis.reset(); });
      });
    }
    return this;
  };

  return model;
})();